@@ -27,7 +27,7 @@
 static xmlNode *handle_aim_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling x-aim attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"IM-AIM", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"IM-AIM", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -35,7 +35,7 @@ static xmlNode *handle_aim_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_manager_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Manager attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Manager", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Manager", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -43,7 +43,7 @@ static xmlNode *handle_manager_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_assistant_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Assistant attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Assistant", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Assistant", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -51,7 +51,7 @@ static xmlNode *handle_assistant_attribute(xmlNode *root, VFormatAttribute *attr
 static xmlNode *handle_anniversary_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Anniversary attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Anniversary", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Anniversary", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -59,7 +59,15 @@ static xmlNode *handle_anniversary_attribute(xmlNode *root, VFormatAttribute *at
 static xmlNode *handle_spouse_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Spouse attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Spouse", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Spouse", NULL);
+	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
+	return current;
+}
+
+static xmlNode *handle_blog_url_attribute(xmlNode *root, VFormatAttribute *attr)
+{
+	osync_trace(TRACE_INTERNAL, "Handling Blog Url attribute");
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"BlogUrl", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -67,7 +75,7 @@ static xmlNode *handle_spouse_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_yahoo_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling IM-Yahoo attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"IM-Yahoo", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"IM-Yahoo", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -75,7 +83,7 @@ static xmlNode *handle_yahoo_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_icq_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling IM-ICQ attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"IM-ICQ", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"IM-ICQ", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -83,7 +91,7 @@ static xmlNode *handle_icq_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_jabber_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Jabber attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"IM-Jabber", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"IM-Jabber", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -91,7 +99,7 @@ static xmlNode *handle_jabber_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_msn_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling MSN attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"IM-MSN", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"IM-MSN", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -103,7 +111,7 @@ static xmlNode *handle_department_attribute(xmlNode *root, VFormatAttribute *att
 	
 	//We need to check first if the node already exists.
 	if (!(current = osxml_get_node(root, "Organization")))
-		current = xmlNewChild(root, NULL, (xmlChar*)"Organization", NULL);
+		current = xmlNewTextChild(root, NULL, (xmlChar*)"Organization", NULL);
 	
 	osxml_node_add(current, "Department", vformat_attribute_get_nth_value(attr, 0));
 	return current;
@@ -116,7 +124,7 @@ static xmlNode *handle_office_attribute(xmlNode *root, VFormatAttribute *attr)
 	
 	//We need to check first if the node already exists.
 	if (!(current = osxml_get_node(root, "Organization")))
-		current = xmlNewChild(root, NULL, (xmlChar*)"Organization", NULL);
+		current = xmlNewTextChild(root, NULL, (xmlChar*)"Organization", NULL);
 	
 	osxml_node_add(current, "Unit", vformat_attribute_get_nth_value(attr, 0));
 	return current;
@@ -125,7 +133,7 @@ static xmlNode *handle_office_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_profession_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling profession attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Profession", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Profession", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -133,7 +141,7 @@ static xmlNode *handle_profession_attribute(xmlNode *root, VFormatAttribute *att
 static xmlNode *handle_gadu_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling gadu attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"IM-GaduGadu", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"IM-GaduGadu", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -141,7 +149,7 @@ static xmlNode *handle_gadu_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_irc_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling IRC attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"IRC", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"IRC", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -149,7 +157,7 @@ static xmlNode *handle_irc_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_sms_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling SMS attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"SMS", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"SMS", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -161,7 +169,7 @@ static xmlNode *handle_organization_attribute(xmlNode *root, VFormatAttribute *a
 	
 	//We need to check first if the node already exists.
 	if (!(current = osxml_get_node(root, "Organization")))
-		current = xmlNewChild(root, NULL, (xmlChar*)"Organization", NULL);
+		current = xmlNewTextChild(root, NULL, (xmlChar*)"Organization", NULL);
 	
 	osxml_node_add(current, "Name", vformat_attribute_get_nth_value(attr, 0));
 	osxml_node_add(current, "Department", vformat_attribute_get_nth_value(attr, 1));
@@ -179,7 +187,7 @@ static xmlNode *handle_organization_attribute(xmlNode *root, VFormatAttribute *a
 static xmlNode *handle_x_kde_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling X-KDE attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"KDE-Extension", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"KDE-Extension", NULL);
 	osxml_node_add(current, "ExtName", vformat_attribute_get_name(attr));
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
@@ -204,6 +212,7 @@ static osync_bool init_kde_to_xml(void *input)
 	g_hash_table_insert(table, "X-KADDRESSBOOK-X-Office", handle_office_attribute);
 	g_hash_table_insert(table, "X-KADDRESSBOOK-X-Profession", handle_profession_attribute);
 	g_hash_table_insert(table, "X-KADDRESSBOOK-X-SpousesName", handle_spouse_attribute);
+	g_hash_table_insert(table, "X-KADDRESSBOOK-BlogFeed", handle_blog_url_attribute);
 	g_hash_table_insert(table, "X-messaging/yahoo-All", handle_yahoo_attribute);
 	g_hash_table_insert(table, "X-messaging/icq-All", handle_icq_attribute);
 	g_hash_table_insert(table, "X-messaging/aim-All", handle_aim_attribute);
@@ -246,22 +255,28 @@ static osync_bool needs_charset(const unsigned char *tmp)
 	return FALSE;
 }
 
+static void add_value_decoded(VFormatAttribute *attr, const char *value, const char *encoding)
+{
+	if (needs_charset((unsigned char*)value))
+		if (!vformat_attribute_has_param (attr, "CHARSET"))
+			vformat_attribute_add_param_with_value(attr, "CHARSET", "UTF-8");
+	
+	if (needs_encoding((unsigned char*)value, encoding)) {
+		if (!vformat_attribute_has_param (attr, "ENCODING"))
+			vformat_attribute_add_param_with_value(attr, "ENCODING", encoding);
+		vformat_attribute_add_value_decoded(attr, value, strlen(value) + 1);
+	} else
+		vformat_attribute_add_value(attr, value);
+}
+
 static void add_value(VFormatAttribute *attr, xmlNode *parent, const char *name, const char *encoding)
 {
 	char *tmp = osxml_find_node(parent, name);
 	if (!tmp)
 		return;
-	
-	if (needs_charset((unsigned char*)tmp))
-		if (!vformat_attribute_has_param (attr, "CHARSET"))
-			vformat_attribute_add_param_with_value(attr, "CHARSET", "UTF-8");
-	
-	if (needs_encoding((unsigned char*)tmp, encoding)) {
-		if (!vformat_attribute_has_param (attr, "ENCODING"))
-			vformat_attribute_add_param_with_value(attr, "ENCODING", encoding);
-		vformat_attribute_add_value_decoded(attr, tmp, strlen(tmp) + 1);
-	} else
-		vformat_attribute_add_value(attr, tmp);
+
+	add_value_decoded(attr, tmp, encoding);	
+
 	g_free(tmp);
 }
 
@@ -301,6 +316,15 @@ static VFormatAttribute *handle_xml_spouse_attribute(VFormat *vcard, xmlNode *ro
 	return attr;
 }
 
+static VFormatAttribute *handle_xml_blog_url_attribute(VFormat *vcard, xmlNode *root, const char *encoding)
+{
+	osync_trace(TRACE_INTERNAL, "Handling blog_url xml attribute");
+	VFormatAttribute *attr = vformat_attribute_new(NULL, "X-KADDRESSBOOK-BlogFeed");
+	add_value(attr, root, "Content", encoding);
+	vformat_add_attribute(vcard, attr);
+	return attr;
+}
+
 static VFormatAttribute *handle_xml_yahoo_attribute(VFormat *vcard, xmlNode *root, const char *encoding)
 {
 	osync_trace(TRACE_INTERNAL, "Handling yahoo xml attribute");
@@ -359,7 +383,7 @@ static VFormatAttribute *handle_xml_organization_attribute(VFormat *vcard, xmlNo
 		char *content = (char*)xmlNodeGetContent(root);
 		if (!strcmp((char*)root->name, "Name")) {
 			org = vformat_attribute_new(NULL, "ORG");
-			vformat_attribute_add_value(org, content);
+			add_value_decoded(org, content, encoding);
 			vformat_add_attribute(vcard, org);
 		}
 		
@@ -445,6 +469,7 @@ static osync_bool init_xml_to_kde(void *input)
 	g_hash_table_insert(hooks->attributes, "Organization", handle_xml_organization_attribute);
 	g_hash_table_insert(hooks->attributes, "Profession", handle_xml_profession_attribute);
 	g_hash_table_insert(hooks->attributes, "Spouse", handle_xml_spouse_attribute);
+	g_hash_table_insert(hooks->attributes, "BlogUrl", handle_xml_blog_url_attribute);
 	g_hash_table_insert(hooks->attributes, "IM-Yahoo", handle_xml_yahoo_attribute);
 	g_hash_table_insert(hooks->attributes, "IM-ICQ", handle_xml_icq_attribute);
 	g_hash_table_insert(hooks->attributes, "IM-AIM", handle_xml_aim_attribute);
