@@ -26,7 +26,7 @@
 static void handle_unknown_parameter(xmlNode *current, VFormatParam *param)
 {
 	osync_trace(TRACE_INTERNAL, "Handling unknown parameter %s", vformat_attribute_param_get_name(param));
-	xmlNode *property = xmlNewChild(current, NULL, (xmlChar*)"UnknownParam",
+	xmlNode *property = xmlNewTextChild(current, NULL, (xmlChar*)"UnknownParam",
 		(xmlChar*)vformat_attribute_param_get_nth_value(param, 0));
 	osxml_node_add(property, "ParamName", vformat_attribute_param_get_name(param));
 }
@@ -37,15 +37,25 @@ static void handle_type_parameter(xmlNode *current, VFormatParam *param)
 	
 	GList *v = vformat_attribute_param_get_values(param);
 	for (; v; v = v->next) {
-		xmlNewChild(current, NULL, (xmlChar*)"Type", (xmlChar*)v->data);
+		xmlNewTextChild(current, NULL, (xmlChar*)"Type", (xmlChar*)v->data);
 	}
 	
 }
 
-static xmlNode *handle_fullname_attribute(xmlNode *root, VFormatAttribute *attr)
+static void handle_value_parameter(xmlNode *current, VFormatParam *param)
 {
-	osync_trace(TRACE_INTERNAL, "Handling fullname attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"FullName", NULL);
+	osync_trace(TRACE_INTERNAL, "Handling value parameter %s", vformat_attribute_param_get_name(param));
+	
+	GList *v = vformat_attribute_param_get_values(param);
+	for (; v; v = v->next) {
+		xmlNewTextChild(current, NULL, (xmlChar*)"Value", (xmlChar*)v->data);
+	}
+}
+
+static xmlNode *handle_formatted_name_attribute(xmlNode *root, VFormatAttribute *attr)
+{
+	osync_trace(TRACE_INTERNAL, "Handling formatted name attribute");
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"FormattedName", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -53,7 +63,7 @@ static xmlNode *handle_fullname_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_name_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling name attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Name", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Name", NULL);
 	osxml_node_add(current, "LastName", vformat_attribute_get_nth_value(attr, 0));
 	osxml_node_add(current, "FirstName", vformat_attribute_get_nth_value(attr, 1));
 	osxml_node_add(current, "Additional", vformat_attribute_get_nth_value(attr, 2));
@@ -62,26 +72,41 @@ static xmlNode *handle_name_attribute(xmlNode *root, VFormatAttribute *attr)
 	return current;
 }
 
+static xmlNode *handle_agent_attribute(xmlNode *root, VFormatAttribute *attr)
+{
+	osync_trace(TRACE_INTERNAL, "Handling agent attribute");
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Agent", NULL);
+	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
+	return current;
+}
+
 static xmlNode *handle_photo_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling photo attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Photo", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Photo", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
 
 static xmlNode *handle_birthday_attribute(xmlNode *root, VFormatAttribute *attr)
 {
+	char *datestamp;
+	const char *tmp;
+
 	osync_trace(TRACE_INTERNAL, "Handling birthday attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Birthday", NULL);
-	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Birthday", NULL);
+	tmp = vformat_attribute_get_nth_value(attr, 0);
+	datestamp = osync_time_datestamp(tmp);
+	osxml_node_add(current, "Content", datestamp);
+	free(datestamp);
+
 	return current;
 }
 
 static xmlNode *handle_address_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling address attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Address", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Address", NULL);
 	osxml_node_add(current, "PostalBox", vformat_attribute_get_nth_value(attr, 0));
 	osxml_node_add(current, "ExtendedAddress", vformat_attribute_get_nth_value(attr, 1));
 	osxml_node_add(current, "Street", vformat_attribute_get_nth_value(attr, 2));
@@ -95,7 +120,7 @@ static xmlNode *handle_address_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_label_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling AddressLabel attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"AddressLabel", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"AddressLabel", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -103,7 +128,7 @@ static xmlNode *handle_label_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_telephone_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Telephone attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Telephone", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Telephone", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -111,7 +136,7 @@ static xmlNode *handle_telephone_attribute(xmlNode *root, VFormatAttribute *attr
 static xmlNode *handle_email_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling EMail attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"EMail", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"EMail", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -119,7 +144,7 @@ static xmlNode *handle_email_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_mailer_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Mailer attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Mailer", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Mailer", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -127,7 +152,7 @@ static xmlNode *handle_mailer_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_timezone_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Timezone attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Timezone", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Timezone", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -135,7 +160,7 @@ static xmlNode *handle_timezone_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_location_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Location attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Location", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Location", NULL);
 	osxml_node_add(current, "Latitude", vformat_attribute_get_nth_value(attr, 0));
 	osxml_node_add(current, "Longitude", vformat_attribute_get_nth_value(attr, 1));
 	return current;
@@ -144,7 +169,7 @@ static xmlNode *handle_location_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_title_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Title attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Title", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Title", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -152,7 +177,7 @@ static xmlNode *handle_title_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_role_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Role attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Role", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Role", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -160,7 +185,7 @@ static xmlNode *handle_role_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_logo_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Logo attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Logo", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Logo", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -168,7 +193,7 @@ static xmlNode *handle_logo_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_organization_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Organization attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Organization", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Organization", NULL);
 	osxml_node_add(current, "Name", vformat_attribute_get_nth_value(attr, 0));
 	osxml_node_add(current, "Department", vformat_attribute_get_nth_value(attr, 1));
 	
@@ -185,23 +210,29 @@ static xmlNode *handle_organization_attribute(xmlNode *root, VFormatAttribute *a
 static xmlNode *handle_note_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Note attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Note", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Note", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
 
 static xmlNode *handle_revision_attribute(xmlNode *root, VFormatAttribute *attr)
 {
+	const char *tmp;
+	char *revision;
+
 	osync_trace(TRACE_INTERNAL, "Handling Revision attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Revision", NULL);
-	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Revision", NULL);
+	tmp = vformat_attribute_get_nth_value(attr, 0);
+	revision = osync_time_timestamp(tmp);
+	osxml_node_add(current, "Content", revision);
+	free(revision);
 	return current;
 }
 
 static xmlNode *handle_sound_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Sound attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Sound", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Sound", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -209,7 +240,7 @@ static xmlNode *handle_sound_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_url_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Url attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Url", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Url", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -217,7 +248,7 @@ static xmlNode *handle_url_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_uid_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Uid attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Uid", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Uid", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -225,7 +256,7 @@ static xmlNode *handle_uid_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_key_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Key attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Key", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Key", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -233,7 +264,7 @@ static xmlNode *handle_key_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_nickname_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Nickname attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Nickname", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Nickname", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -241,7 +272,7 @@ static xmlNode *handle_nickname_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_class_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Class attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Class", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Class", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -249,7 +280,7 @@ static xmlNode *handle_class_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_categories_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Categories attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Categories", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Categories", NULL);
 	
 	GList *values = vformat_attribute_get_values_decoded(attr);
 	for (; values; values = values->next) {
@@ -264,7 +295,7 @@ static xmlNode *handle_categories_attribute(xmlNode *root, VFormatAttribute *att
 static xmlNode *handle_unknown_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling unknown attribute %s", vformat_attribute_get_name(attr));
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"UnknownNode", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"UnknownNode", NULL);
 	osxml_node_add(current, "NodeName", vformat_attribute_get_name(attr));
 	GList *values = vformat_attribute_get_values_decoded(attr);
 	for (; values; values = values->next) {
@@ -339,16 +370,83 @@ has_value:;
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
+static void _generate_formatted_name(VFormat *vcard, xmlNode *root)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, root);
+	VFormatAttribute *n = vformat_find_attribute(vcard, "N");
+	GList *v = vformat_attribute_get_values(n);
+	GString *fnentry;
+	fnentry = g_string_new("");
+
+	// NAME:LAST;FIRST;ADDITIONAL;PREFIX;SUFFIX
+	// FN:PREFIX FIRST ADDITIONAL LAST SUFFIX
+
+	int order[5] = {3, 1, 2, 0, 4};
+	int i = 0;
+	char *str = NULL;
+	for (i = 0; i < 5; i++) {
+		if ((str = g_list_nth_data(v, order[i])) && str[0]) {
+			if (fnentry->len != 0)
+				g_string_append(fnentry, " ");
+			g_string_append(fnentry, str);
+		}
+	}
+	
+	osync_trace(TRACE_INTERNAL, "Handling formattedname attribute");
+
+	if (fnentry->len != 0) {
+		xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"FormattedName", NULL);
+		osxml_node_add(current, "Content", fnentry->str);
+	} else {
+		osync_trace(TRACE_INTERNAL, "FN is empty!");
+	}	
+	
+	g_string_free(fnentry,TRUE);
+	osync_trace(TRACE_EXIT, "%s", __func__);
+	return;
+}
+
+static void _generate_name_from_fn(VFormat *vcard, xmlNode *root)
+{
+	/*
+	 * We copy FN to N:LASTNAME because we don't now how FN was build.
+	 * e.g. we don't know if FN is "PREFIX FIRST LAST" or "FIRST ADDITIONAL LAST"
+	 * With copying FN to N we prevent the vcard from being invalid.
+	 */
+
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, root);
+	VFormatAttribute *n = vformat_find_attribute(vcard, "FN");
+	char *fn = vformat_attribute_get_value(n);
+
+        osync_trace(TRACE_INTERNAL, "Handling name attribute");
+	if (strlen(fn) != 0) {
+		xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Name", NULL);
+        	osxml_node_add(current, "LastName", fn);
+	} else {
+		osync_trace(TRACE_INTERNAL, "Name is empty");
+	}
+		
+	osync_trace(TRACE_EXIT, "%s", __func__);
+	return;
+}	
+
 static osync_bool conv_vcard_to_xml(void *conv_data, char *input, int inpsize, char **output, int *outpsize, osync_bool *free_input, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %i, %p, %p, %p, %p)", __func__, conv_data, input, inpsize, output, outpsize, free_input, error);
 	
 	GHashTable *hooks = (GHashTable *)conv_data;
 	
-	osync_trace(TRACE_INTERNAL, "Input Vcard is:\n%s", input);
+	osync_trace(TRACE_SENSITIVE, "Input Vcard is:\n%s", input);
 	
+	/* The input is not null-terminated, but vformat_new_from_string() expects a null-terminated string */
+	char *input_str = g_malloc(inpsize + 1);
+	memcpy(input_str, input, inpsize);
+	input_str[inpsize] = '\0';
+
 	//Parse the vcard
-	VFormat *vcard = vformat_new_from_string(input);
+	VFormat *vcard = vformat_new_from_string(input_str);
+
+	g_free(input_str);
 	
 	osync_trace(TRACE_INTERNAL, "Creating xml doc");
 	
@@ -365,8 +463,20 @@ static osync_bool conv_vcard_to_xml(void *conv_data, char *input, int inpsize, c
 		VFormatAttribute *attr = a->data;
 		vcard_handle_attribute(hooks, root, attr);
 	}
-	
-	osync_trace(TRACE_INTERNAL, "Output XML is:\n%s", osxml_write_to_string(doc));
+
+	//Generate FormattedName from Name if it doesn't exist
+	if (!vformat_find_attribute(vcard, "FN") && vformat_find_attribute(vcard, "N")) {
+		_generate_formatted_name(vcard,root);
+	}
+
+	//Generate Name from FormattedName if it doesn't exist
+	if (!vformat_find_attribute(vcard, "N") && vformat_find_attribute(vcard, "FN")) {
+		_generate_name_from_fn(vcard,root);
+	}		
+
+	xmlChar *str = osxml_write_to_string(doc);
+	osync_trace(TRACE_SENSITIVE, "Output XML is:\n%s", str);
+	xmlFree(str);
 	
 	*free_input = TRUE;
 	*output = (char *)doc;
@@ -410,7 +520,7 @@ static void add_value(VFormatAttribute *attr, xmlNode *parent, const char *name,
 		 * This is necessary because some fields (N and ADR, for example) need
 		 * a specific order of the values
 		 */
-		tmp = "";
+		tmp = g_strdup("");
 	}
 	
 	if (needs_charset((unsigned char*)tmp))
@@ -436,6 +546,16 @@ static void handle_xml_type_parameter(VFormatAttribute *attr, xmlNode *current)
 	g_free(content);
 }
 
+static void handle_xml_value_parameter(VFormatAttribute *attr, xmlNode *current)
+{
+	osync_trace(TRACE_INTERNAL, "Handling value xml parameter");
+	char *content = (char*)xmlNodeGetContent(current);
+	VFormatParam *param = vformat_attribute_param_new("VALUE");
+	vformat_attribute_param_add_value(param, content);
+	vformat_attribute_add_param (attr, param);
+	g_free(content);
+}
+
 static void handle_xml_category_parameter(VFormatAttribute *attr, xmlNode *current)
 {
 	osync_trace(TRACE_INTERNAL, "Handling category xml parameter");
@@ -495,9 +615,9 @@ static VFormatAttribute *xml_handle_unknown_attribute(VFormat *vcard, xmlNode *r
 	return attr;
 }
 
-static VFormatAttribute *handle_xml_fullname_attribute(VFormat *vcard, xmlNode *root, const char *encoding)
+static VFormatAttribute *handle_xml_formatted_name_attribute(VFormat *vcard, xmlNode *root, const char *encoding)
 {
-	osync_trace(TRACE_INTERNAL, "Handling fullname xml attribute");
+	osync_trace(TRACE_INTERNAL, "Handling formatted name xml attribute");
 	VFormatAttribute *attr = vformat_attribute_new(NULL, "FN");
 	add_value(attr, root, "Content", encoding);
 	vformat_add_attribute(vcard, attr);
@@ -506,7 +626,7 @@ static VFormatAttribute *handle_xml_fullname_attribute(VFormat *vcard, xmlNode *
 
 static VFormatAttribute *handle_xml_name_attribute(VFormat *vcard, xmlNode *root, const char *encoding)
 {
-	osync_trace(TRACE_INTERNAL, "Handling fullname xml attribute");
+	osync_trace(TRACE_INTERNAL, "Handling name xml attribute");
 	VFormatAttribute *attr = vformat_attribute_new(NULL, "N");
 	add_value(attr, root, "LastName", encoding);
 	add_value(attr, root, "FirstName", encoding);
@@ -517,6 +637,15 @@ static VFormatAttribute *handle_xml_name_attribute(VFormat *vcard, xmlNode *root
 	return attr;
 }
 
+static VFormatAttribute *handle_xml_agent_attribute(VFormat *vcard, xmlNode *root, const char *encoding)
+{
+	osync_trace(TRACE_INTERNAL, "Handling agent xml attribute");
+	VFormatAttribute *attr = vformat_attribute_new(NULL, "AGENT");
+	add_value(attr, root, "Content", encoding);
+	vformat_add_attribute(vcard, attr);
+	return attr;
+}
+
 static void xml_vcard_handle_attribute(OSyncHookTables *hooks, VFormat *vcard, xmlNode *root, const char *encoding)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p:%s)", __func__, hooks, vcard, root, root ? (char *)root->name : "None");
@@ -760,7 +889,9 @@ static osync_bool conv_xml_to_vcard(void *user_data, char *input, int inpsize, c
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %i, %p, %p, %p, %p)", __func__, user_data, input, inpsize, output, outpsize, free_input, error);
 	
-	osync_trace(TRACE_INTERNAL, "Input XML is:\n%s", osxml_write_to_string((xmlDoc *)input));
+	xmlChar *str = osxml_write_to_string((xmlDoc *)input);
+	osync_trace(TRACE_SENSITIVE, "Input XML is:\n%s", str);
+	xmlFree(str);
 	
 	//Get the root node of the input document
 	xmlNode *root = xmlDocGetRootElement((xmlDoc *)input);
@@ -795,8 +926,8 @@ static osync_bool conv_xml_to_vcard(void *user_data, char *input, int inpsize, c
 	
 	*free_input = TRUE;
 	*output = vformat_to_string(vcard, target);
-	osync_trace(TRACE_INTERNAL, "vcard output is: \n%s", *output);
-	*outpsize = strlen(*output) + 1;
+	osync_trace(TRACE_SENSITIVE, "vcard output is: \n%s", *output);
+	*outpsize = strlen(*output);
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	
 	return TRUE;
@@ -822,7 +953,7 @@ static OSyncConvCmpResult compare_contact(OSyncChange *leftchange, OSyncChange *
 	{100, "/contact/Name"},
 	//{20, "/contact/Telephone"},
 	//{20, "/contact/Address"},
-	//{1, "/contact/UnknownNode"},
+	{0, "/contact/UnknownNode"},
 	{0, "/contact/*/Slot"},
 	{0, "/contact/*/Type"},
 	{0, "/contact/WantsHtml"},
@@ -843,7 +974,7 @@ static char *print_contact(OSyncChange *change)
 {
 	xmlDoc *doc = (xmlDoc *)osync_change_get_data(change);
 	
-	return osxml_write_to_string(doc);
+	return (char *)osxml_write_to_string(doc);
 }
 
 static void destroy_xml(char *data, size_t size)
@@ -856,8 +987,9 @@ static void *init_vcard_to_xml(void)
 	osync_trace(TRACE_ENTRY, "%s", __func__);
 	GHashTable *table = g_hash_table_new(g_str_hash, g_str_equal);
 	
-	g_hash_table_insert(table, "FN", handle_fullname_attribute);
+	g_hash_table_insert(table, "FN", handle_formatted_name_attribute);
 	g_hash_table_insert(table, "N", handle_name_attribute);
+	g_hash_table_insert(table, "AGENT", handle_agent_attribute);
 	g_hash_table_insert(table, "PHOTO", handle_photo_attribute);
 	g_hash_table_insert(table, "BDAY", handle_birthday_attribute);
 	g_hash_table_insert(table, "ADR", handle_address_attribute);
@@ -889,6 +1021,9 @@ static void *init_vcard_to_xml(void)
 	g_hash_table_insert(table, "CHARSET", HANDLE_IGNORE);
 	
 	g_hash_table_insert(table, "TYPE", handle_type_parameter);
+	g_hash_table_insert(table, "VALUE", handle_value_parameter);
+
+	g_hash_table_insert(table, "X-IRMC-LUID", HANDLE_IGNORE);
 	
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, table);
 	return (void *)table;
@@ -908,8 +1043,9 @@ static void *init_xml_to_vcard(void)
 	hooks->attributes = g_hash_table_new(g_str_hash, g_str_equal);
 	hooks->parameters = g_hash_table_new(g_str_hash, g_str_equal);
 	
-	g_hash_table_insert(hooks->attributes, "FullName", handle_xml_fullname_attribute);
+	g_hash_table_insert(hooks->attributes, "FormattedName", handle_xml_formatted_name_attribute);
 	g_hash_table_insert(hooks->attributes, "Name", handle_xml_name_attribute);
+	g_hash_table_insert(hooks->attributes, "Agent", handle_xml_agent_attribute);
 	g_hash_table_insert(hooks->attributes, "Photo", handle_xml_photo_attribute);
 	g_hash_table_insert(hooks->attributes, "Birthday", handle_xml_birthday_attribute);
 	g_hash_table_insert(hooks->attributes, "Address", handle_xml_address_attribute);
@@ -935,6 +1071,7 @@ static void *init_xml_to_vcard(void)
 	g_hash_table_insert(hooks->attributes, "UnknownNode", xml_handle_unknown_attribute);
 	
 	g_hash_table_insert(hooks->parameters, "Type", handle_xml_type_parameter);
+	g_hash_table_insert(hooks->parameters, "Value", handle_xml_value_parameter);
 	g_hash_table_insert(hooks->parameters, "Category", handle_xml_category_parameter);
 	g_hash_table_insert(hooks->parameters, "Unit", handle_xml_unit_parameter);
 	
@@ -946,7 +1083,7 @@ static void *init_xml_to_vcard(void)
 
 static void fin_xml_to_vcard(void *data)
 {
-	OSyncHookTables *hooks = (OSyncHookTables *)hooks;
+	OSyncHookTables *hooks = (OSyncHookTables *)data;
 	g_hash_table_destroy(hooks->attributes);
 	g_hash_table_destroy(hooks->parameters);
 	g_free(hooks);
@@ -988,6 +1125,8 @@ void get_info(OSyncEnv *env)
 	osync_env_format_set_print_func(env, "xml-contact", print_contact);
 	osync_env_format_set_copy_func(env, "xml-contact", osxml_copy);
 	osync_env_format_set_revision_func(env, "xml-contact", get_revision);
+	osync_env_format_set_marshall_func(env, "xml-contact", osxml_marshall);
+	osync_env_format_set_demarshall_func(env, "xml-contact", osxml_demarshall);
 	
 	osync_env_register_converter(env, CONVERTER_CONV, "vcard21", "xml-contact", conv_vcard_to_xml);
 	osync_env_converter_set_init(env, "vcard21", "xml-contact", init_vcard_to_xml, fin_vcard_to_xml);
