@@ -27,23 +27,29 @@
 static void handle_unknown_parameter(xmlNode *current, VFormatParam *param)
 {
 	osync_trace(TRACE_INTERNAL, "Handling unknown parameter %s", vformat_attribute_param_get_name(param));
-	xmlNode *property = xmlNewChild(current, NULL, (xmlChar*)"UnknownParam",
+	xmlNode *property = xmlNewTextChild(current, NULL, (xmlChar*)"UnknownParam",
 		(xmlChar*)vformat_attribute_param_get_nth_value(param, 0));
 	osxml_node_add(property, "ParamName", vformat_attribute_param_get_name(param));
 }
 
 static xmlNode *handle_created_attribute(xmlNode *root, VFormatAttribute *attr)
 {
+	char *timestamp;
+	const char *tmp;
+
 	osync_trace(TRACE_INTERNAL, "Handling created attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"DateCreated", NULL);
-	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"DateCreated", NULL);
+	tmp = vformat_attribute_get_nth_value(attr, 0); 
+	timestamp = osync_time_timestamp(tmp);
+	osxml_node_add(current, "Content", timestamp);
+	g_free(timestamp);
 	return current;
 }
 
 static xmlNode *handle_last_modified_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling last_modified attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"LastModified", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"LastModified", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -51,7 +57,7 @@ static xmlNode *handle_last_modified_attribute(xmlNode *root, VFormatAttribute *
 static xmlNode *handle_summary_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling summary attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Summary", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Summary", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -59,7 +65,7 @@ static xmlNode *handle_summary_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_categories_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Categories attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Categories", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Categories", NULL);
 	
 	GList *values = vformat_attribute_get_values_decoded(attr);
 	for (; values; values = values->next) {
@@ -74,7 +80,7 @@ static xmlNode *handle_categories_attribute(xmlNode *root, VFormatAttribute *att
 static xmlNode *handle_body_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling body attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Body", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Body", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -82,7 +88,7 @@ static xmlNode *handle_body_attribute(xmlNode *root, VFormatAttribute *attr)
 static xmlNode *handle_class_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Class attribute");
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"Class", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"Class", NULL);
 	osxml_node_add(current, "Content", vformat_attribute_get_nth_value(attr, 0));
 	return current;
 }
@@ -90,14 +96,14 @@ static xmlNode *handle_class_attribute(xmlNode *root, VFormatAttribute *attr)
 static void handle_type_parameter(xmlNode *current, VFormatParam *param)
 {
 	osync_trace(TRACE_INTERNAL, "Handling type parameter %s", vformat_attribute_param_get_name(param));
-	xmlNewChild(current, NULL, (xmlChar*)"Type",
+	xmlNewTextChild(current, NULL, (xmlChar*)"Type",
 		(xmlChar*)vformat_attribute_param_get_nth_value(param, 0));
 }
 
 static xmlNode *handle_unknown_attribute(xmlNode *root, VFormatAttribute *attr)
 {
 	osync_trace(TRACE_INTERNAL, "Handling unknown attribute %s", vformat_attribute_get_name(attr));
-	xmlNode *current = xmlNewChild(root, NULL, (xmlChar*)"UnknownNode", NULL);
+	xmlNode *current = xmlNewTextChild(root, NULL, (xmlChar*)"UnknownNode", NULL);
 	osxml_node_add(current, "NodeName", vformat_attribute_get_name(attr));
 	GList *values = vformat_attribute_get_values_decoded(attr);
 	for (; values; values = values->next) {
@@ -178,10 +184,17 @@ static osync_bool conv_vnote_to_xml(void *conv_data, char *input, int inpsize, c
 	
 	GHashTable *hooks = (GHashTable *)conv_data;
 	
-	osync_trace(TRACE_INTERNAL, "Input vnote is:\n%s", input);
+	osync_trace(TRACE_SENSITIVE, "Input vnote is:\n%s", input);
 	
+	/* The input is not null-terminated, but vformat_new_from_string() expects a null-terminated string */
+	char *input_str = g_malloc(inpsize + 1);
+	memcpy(input_str, input, inpsize);
+	input_str[inpsize] = '\0';
+
 	//Parse the vnote
-	VFormat *vnote = vformat_new_from_string(input);
+	VFormat *vnote = vformat_new_from_string(input_str);
+
+	g_free(input_str);
 	
 	osync_trace(TRACE_INTERNAL, "Creating xml doc");
 	
@@ -199,7 +212,9 @@ static osync_bool conv_vnote_to_xml(void *conv_data, char *input, int inpsize, c
 		vnote_handle_attribute(hooks, root, attr);
 	}
 	
-	osync_trace(TRACE_INTERNAL, "Output XML is:\n%s", osxml_write_to_string(doc));
+	xmlChar *str = osxml_write_to_string(doc);
+	osync_trace(TRACE_SENSITIVE, "Output XML is:\n%s", str);
+	xmlFree(str);
 	
 	*free_input = TRUE;
 	*output = (char *)doc;
@@ -393,7 +408,9 @@ static osync_bool conv_xml_to_vnote(void *user_data, char *input, int inpsize, c
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %i, %p, %p, %p, %p)", __func__, user_data, input, inpsize, output, outpsize, free_input, error);
 	
-	osync_trace(TRACE_INTERNAL, "Input XML is:\n%s", osxml_write_to_string((xmlDoc *)input));
+	xmlChar *str = osxml_write_to_string((xmlDoc *)input);
+	osync_trace(TRACE_SENSITIVE, "Input XML is:\n%s", str);
+	xmlFree(str);
 	
 	//Get the root node of the input document
 	xmlNode *root = osxml_node_get_root((xmlDoc *)input, "Note", error);
@@ -414,8 +431,8 @@ static osync_bool conv_xml_to_vnote(void *user_data, char *input, int inpsize, c
 	
 	*free_input = TRUE;
 	*output = vformat_to_string(vnote, VFORMAT_NOTE);
-	osync_trace(TRACE_INTERNAL, "vnote output is: \n%s", *output);
-	*outpsize = strlen(*output) + 1;
+	osync_trace(TRACE_SENSITIVE, "vnote output is: \n%s", *output);
+	*outpsize = strlen(*output);
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	
 	return TRUE;
@@ -446,7 +463,7 @@ static char *print_note(OSyncChange *change)
 {
 	xmlDoc *doc = (xmlDoc *)osync_change_get_data(change);
 	
-	return osxml_write_to_string(doc);
+	return (char *)osxml_write_to_string(doc);
 }
 
 static void destroy_xml(char *data, size_t size)
@@ -513,7 +530,7 @@ static void *init_xml_to_vnote(void)
 
 static void fin_xml_to_vnote(void *data)
 {
-	OSyncHookTables *hooks = (OSyncHookTables *)hooks;
+	OSyncHookTables *hooks = (OSyncHookTables *)data;
 	g_hash_table_destroy(hooks->attributes);
 	g_hash_table_destroy(hooks->parameters);
 	g_free(hooks);
@@ -555,6 +572,8 @@ void get_info(OSyncEnv *env)
 	osync_env_format_set_print_func(env, "xml-note", print_note);
 	osync_env_format_set_copy_func(env, "xml-note", osxml_copy);
 	osync_env_format_set_revision_func(env, "xml-note", get_revision);
+	osync_env_format_set_marshall_func(env, "xml-note", osxml_marshall);
+	osync_env_format_set_demarshall_func(env, "xml-note", osxml_demarshall);
 	
 	osync_env_register_converter(env, CONVERTER_CONV, "vnote11", "xml-note", conv_vnote_to_xml);
 	osync_env_converter_set_init(env, "vnote11", "xml-note", init_vnote_to_xml, fin_vnote_to_xml);
