@@ -171,7 +171,7 @@ OSyncConvCmpResult osync_change_compare_data(OSyncChange *leftchange, OSyncChang
 	if (!osync_change_convert_to_common(rightchange, &error)) {
 		osync_trace(TRACE_INTERNAL, "osync_change_compare_data: %s", osync_error_print(&error));
 		osync_error_free(&error);
-		osync_trace(TRACE_EXIT, "osync_change_compare_data: MISMATCH: Could not convert leftchange to common format");
+		osync_trace(TRACE_EXIT, "osync_change_compare_data: MISMATCH: Could not convert rightchange to common format");
 		return CONV_DATA_MISMATCH;
 	}
 
@@ -274,8 +274,11 @@ osync_bool osync_change_copy_data(OSyncChange *source, OSyncChange *target, OSyn
 	
 	if (!format || !format->copy_func) {
 		osync_trace(TRACE_INTERNAL, "We cannot copy the change, falling back to memcpy");
-		target->data = g_malloc0(sizeof(char) * source->size);
+		target->data = g_malloc0(sizeof(char) * (source->size + 1));
 		memcpy(target->data, source->data, source->size);
+
+		/* Make sure that its null terminated */
+		target->data[source->size] = 0;
 		target->size = source->size;
 	} else {
 		if (!format->copy_func(source->data, source->size, &(target->data), &(target->size))) {
