@@ -254,6 +254,21 @@ OSyncLockState osync_group_lock(OSyncGroup *group)
 		osync_trace(TRACE_EXIT_ERROR, "osync_group_lock: %s", strerror(errno));
 		return OSYNC_LOCK_STALE;
 	} else {
+
+		/* Set FD_CLOEXEC flags for the lock file descriptor. We don't want the
+		 * subprocesses created by plugins or the engine to keep holding the lock
+		 */
+		int oldflags = fcntl(group->lock_fd, F_GETFD);
+		if (oldflags == -1) {
+			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, "Unable to get fd flags");
+			return OSYNC_LOCK_STALE;
+		}
+
+		if (fcntl(group->lock_fd, F_SETFD, oldflags|FD_CLOEXEC) == -1) {
+			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, "Unable to set fd flags");
+			return OSYNC_LOCK_STALE;
+		}
+
 		if (flock(group->lock_fd, LOCK_EX | LOCK_NB) == -1) {
 			if (errno == EWOULDBLOCK) {
 				osync_debug("GRP", 4, "locking group: is locked2");
@@ -362,7 +377,7 @@ osync_bool osync_group_save(OSyncGroup *group, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, group, error);
 	g_assert(group);
-	osync_assert(group->env, "You must specify a Environment prior to saving the group");
+	osync_assert_msg(group->env, "You must specify a Environment prior to saving the group");
 	
 	if (!group->configdir) {
 		group->id = _osync_env_create_group_id(group->env);
@@ -393,39 +408,39 @@ osync_bool osync_group_save(OSyncGroup *group, OSyncError **error)
 	GList *f;
 	for (f = group->filters; f; f = f->next) {
 		OSyncFilter *filter = f->data;
-		xmlNodePtr child = xmlNewChild(doc->children, NULL, (xmlChar*)"filter", NULL);
+		xmlNodePtr child = xmlNewTextChild(doc->children, NULL, (xmlChar*)"filter", NULL);
 		
 		if (filter->sourcememberid) {
 			char *sourcememberid = g_strdup_printf("%lli", filter->sourcememberid);
-			xmlNewChild(child, NULL, (xmlChar*)"sourcemember", (xmlChar*)sourcememberid);
+			xmlNewTextChild(child, NULL, (xmlChar*)"sourcemember", (xmlChar*)sourcememberid);
 			g_free(sourcememberid);
 		}
 		if (filter->destmemberid) {
 			char *destmemberid = g_strdup_printf("%lli", filter->destmemberid);
-			xmlNewChild(child, NULL, (xmlChar*)"destmember", (xmlChar*)destmemberid);
+			xmlNewTextChild(child, NULL, (xmlChar*)"destmember", (xmlChar*)destmemberid);
 			g_free(destmemberid);
 		}
 		if (filter->sourceobjtype)
-			xmlNewChild(child, NULL, (xmlChar*)"sourceobjtype", (xmlChar*)filter->sourceobjtype);
+			xmlNewTextChild(child, NULL, (xmlChar*)"sourceobjtype", (xmlChar*)filter->sourceobjtype);
 		if (filter->destobjtype)
-			xmlNewChild(child, NULL, (xmlChar*)"destobjtype", (xmlChar*)filter->destobjtype);
+			xmlNewTextChild(child, NULL, (xmlChar*)"destobjtype", (xmlChar*)filter->destobjtype);
 		if (filter->detectobjtype)
-			xmlNewChild(child, NULL, (xmlChar*)"detectobjtype", (xmlChar*)filter->detectobjtype);
+			xmlNewTextChild(child, NULL, (xmlChar*)"detectobjtype", (xmlChar*)filter->detectobjtype);
 		if (filter->action) {
 			char *action = g_strdup_printf("%i", filter->action);
-			xmlNewChild(child, NULL, (xmlChar*)"action", (xmlChar*)action);
+			xmlNewTextChild(child, NULL, (xmlChar*)"action", (xmlChar*)action);
 			g_free(action);
 		}
 		if (filter->function_name)
-			xmlNewChild(child, NULL, (xmlChar*)"function_name", (xmlChar*)filter->function_name);
+			xmlNewTextChild(child, NULL, (xmlChar*)"function_name", (xmlChar*)filter->function_name);
 		if (filter->config)
-			xmlNewChild(child, NULL, (xmlChar*)"config", (xmlChar*)filter->config);
+			xmlNewTextChild(child, NULL, (xmlChar*)"config", (xmlChar*)filter->config);
 	}
 
-	xmlNewChild(doc->children, NULL, (xmlChar*)"groupname", (xmlChar*)group->name);
+	xmlNewTextChild(doc->children, NULL, (xmlChar*)"groupname", (xmlChar*)group->name);
 
 	char *tmstr = g_strdup_printf("%i", (int)group->last_sync);
-	xmlNewChild(doc->children, NULL, (xmlChar*)"last_sync", (xmlChar*)tmstr);
+	xmlNewTextChild(doc->children, NULL, (xmlChar*)"last_sync", (xmlChar*)tmstr);
 	g_free(tmstr);
 
 	xmlSaveFile(filename, doc);
@@ -784,15 +799,21 @@ void osync_group_reset_slow_sync(OSyncGroup *group, const char *objtypestr)
  */
 osync_bool osync_group_get_slow_sync(OSyncGroup *group, const char *objtype)
 {
+	osync_trace(TRACE_ENTRY, "%s(%p, %s)", __func__, group, objtype);
+	
 	g_assert(group);
 	OSyncFormatEnv *env = group->conv_env;
 	g_assert(env);
 	
 	OSyncObjType *osync_objtype = osync_conv_find_objtype(env, "data");
-	if (osync_objtype->needs_slow_sync)
+	if (osync_objtype->needs_slow_sync) {
+		osync_trace(TRACE_EXIT, "%s: Data objtype needs slow-sync", __func__);
 		return TRUE;
+	}
 	osync_objtype = osync_conv_find_objtype(env, objtype);
 	g_assert(osync_objtype);
+	
+	osync_trace(TRACE_EXIT, "%s: %i", __func__, osync_objtype->needs_slow_sync);
 	return osync_objtype->needs_slow_sync;
 }
 
@@ -951,7 +972,7 @@ osync_bool osync_group_remove_changelog(OSyncGroup *group, OSyncChange *change,
 void osync_group_set_last_synchronization(OSyncGroup *group, time_t last_sync)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, not shown)", __func__, last_sync);
-	osync_assert(group, "Group missing");
+	osync_assert_msg(group, "Group missing");
 	
 	group->last_sync = last_sync;
                
@@ -967,7 +988,7 @@ void osync_group_set_last_synchronization(OSyncGroup *group, time_t last_sync)
  */
 time_t osync_group_get_last_synchronization(OSyncGroup *group)
 {
-	osync_assert(group, "Group missing");
+	osync_assert_msg(group, "Group missing");
 	return group->last_sync;
 }
 
