@@ -120,8 +120,9 @@ osync_bool osync_member_get_objtype_sinks(OSyncMember *member, GList **list_ptr,
 osync_bool osync_member_read_config(OSyncMember *member, char **data, int *size, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "osync_member_read_config(%p, %p, %p, %p)", member, data, size, error);
+	
 	if (!osync_member_instance_default_plugin(member, error)) {
-		osync_trace(TRACE_EXIT_ERROR, "osync_member_read_config: %i", osync_error_print(error));
+		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 		return FALSE;
 	}
 	
@@ -175,6 +176,7 @@ OSyncMember *osync_member_new(OSyncGroup *group)
 	}
 	
 	member->memberfunctions = osync_memberfunctions_new();
+	member->name = NULL;
 
 	return member;
 }
@@ -186,7 +188,7 @@ OSyncMember *osync_member_new(OSyncGroup *group)
  */
 void osync_member_free(OSyncMember *member)
 {
-	osync_assert(member, "You must set a member to free");
+	osync_assert_msg(member, "You must set a member to free");
 	
 	if (member->group)
 		osync_group_remove_member(member->group, member);
@@ -396,6 +398,26 @@ void osync_member_set_configdir(OSyncMember *member, const char *configdir)
 	member->configdir = g_strdup(configdir);
 }
 
+osync_bool osync_member_need_config(OSyncMember *member, OSyncConfigurationTypes *type, OSyncError **error)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, member, type, error);
+	g_assert(member);
+	g_assert(type);
+	*type = NO_CONFIGURATION;
+	
+	if (!osync_member_instance_default_plugin(member, error))
+		goto error;
+	
+	*type = member->plugin->info.config_type;
+	
+	osync_trace(TRACE_EXIT, "%s: %i", __func__, *type);
+	return TRUE;
+
+error:
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	return FALSE;
+}
+
 /** @brief Gets the configuration data of this member
  * 
  * The config file is read in this order:
@@ -465,8 +487,7 @@ osync_bool osync_member_get_config(OSyncMember *member, char **data, int *size,
 	g_assert(member);
 	osync_bool ret = TRUE;
 
-	if (!member->plugin) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Member has not instanced a plugin yet");
+	if (!osync_member_instance_default_plugin(member, error)) {
 		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 		return FALSE;
 	}
@@ -604,6 +625,8 @@ OSyncMember *osync_member_load(OSyncGroup *group, const char *path, OSyncError *
 		if (str) {
 			if (!xmlStrcmp(cur->name, (const xmlChar *)"pluginname"))
 				member->pluginname = g_strdup(str);
+			if (!xmlStrcmp(cur->name, (const xmlChar *)"name"))
+				member->name = g_strdup(str);
 			xmlFree(str);
 		}
 		cur = cur->next;
@@ -653,7 +676,9 @@ osync_bool osync_member_save(OSyncMember *member, OSyncError **error)
 	doc = xmlNewDoc((xmlChar*)"1.0");
 	doc->children = xmlNewDocNode(doc, NULL, (xmlChar*)"syncmember", NULL);
 	//The plugin name
-	xmlNewChild(doc->children, NULL, (xmlChar*)"pluginname", (xmlChar*)member->pluginname);
+	xmlNewTextChild(doc->children, NULL, (xmlChar*)"pluginname", (xmlChar*)member->pluginname);
+  //The name
+	xmlNewTextChild(doc->children, NULL, (xmlChar*)"name", (xmlChar*)member->name);
 	xmlSaveFile(filename, doc);
 	xmlFreeDoc(doc);
 	g_free(filename);
@@ -740,11 +765,14 @@ void osync_member_set_slow_sync(OSyncMember *member, const char *objtypestr, osy
  */
 osync_bool osync_member_get_slow_sync(OSyncMember *member, const char *objtypestr)
 {
+	osync_trace(TRACE_ENTRY, "%s(%p, %s)", __func__, member, objtypestr);
 	g_assert(member);	
 	OSyncGroup *group = osync_member_get_group(member);
 	g_assert(group);
 
 	osync_bool needs_slow_sync = osync_group_get_slow_sync(group, objtypestr);
+	
+	osync_trace(TRACE_EXIT, "%s: %i", __func__, needs_slow_sync);
 	return needs_slow_sync;
 }
 
@@ -938,7 +966,14 @@ void osync_member_set_objtype_enabled(OSyncMember *member, const char *objtypest
 			sink->enabled = enabled;
 		}
 	} else {
-		sink = osync_member_find_objtype_sink(member, objtypestr);
+		GList *o = NULL;
+	        for (o = member->objtype_sinks; o; o = o->next) {
+	                sink = o->data;
+	                if (!strcmp(sink->objtype->name, objtypestr))
+	                        break;
+			sink = NULL;
+	        }
+
 		if (!sink) {
 			osync_trace(TRACE_EXIT_ERROR, "Unable to find sink with name \"%s\"", objtypestr);
 			return;
@@ -1072,6 +1107,25 @@ void osync_member_read_change(OSyncMember *member, OSyncChange *change, OSyncEng
 	osync_trace(TRACE_EXIT_ERROR, "%s: Unable to find a sink", __func__);
 }
 
+/** @brief Checks if the member has a read method for the given objtype
+ * 
+ * @param member The member
+ * @param objtype The objtype for which to check the read methid
+ * @return TRUE if the member has read function, FALSE otherwise
+ * 
+ */
+osync_bool osync_member_has_read_function(OSyncMember *member, OSyncObjType *objtype)
+{
+	GList *i;
+	for (i = member->format_sinks; i; i = i->next) {
+		OSyncObjFormatSink *fmtsink = i->data;
+
+		if (osync_objformat_get_objtype(fmtsink->format) == objtype)
+			return fmtsink->functions.read ? TRUE : FALSE;
+	}
+	return FALSE;
+}
+
 /** @brief Gets the "real" data of a object
  * 
  * Calls the get_data function on the plugin
@@ -1185,13 +1239,12 @@ void osync_member_commit_change(OSyncMember *member, OSyncChange *change, OSyncE
 	g_assert(member);
 	g_assert(change);
 
-	OSyncFormatEnv *env = osync_member_get_format_env(member);
 	OSyncContext *context = osync_context_new(member);
 	context->callback_function = function;
 	context->calldata = user_data;
 	
 	
-	OSyncObjType *type = change->objtype;
+	OSyncObjType *type = osync_change_get_objtype(change);
 	
 	/* This is just an optmization:
 	 *
@@ -1213,13 +1266,6 @@ void osync_member_commit_change(OSyncMember *member, OSyncChange *change, OSyncE
 	}
 
 
-	OSyncError *error = NULL;
-	if (!osync_change_convert_member_sink(env, change, member, &error)) {
-		osync_context_report_error(context, OSYNC_ERROR_CONVERT, "Unable to convert change");
-		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
-		osync_error_free(&error);
-		return;
-	}
 	//The destobjtype is the objtype of the format to which
 	//the change was just converted
 	change->destobjtype = g_strdup(osync_change_get_objtype(change)->name);
@@ -1345,6 +1391,18 @@ void osync_member_committed_all(OSyncMember *member, OSyncEngCallback function,
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
+void osync_member_set_name(OSyncMember *member, const char *name)
+{
+	g_assert(member);
+	if (member->name)
+		g_free(member->name);
+	member->name = g_strdup(name);
+}
+
+const char *osync_member_get_name(OSyncMember *member)
+{
+	return member->name;
+}
 
 /** @brief Adds random data to a member
  * 
@@ -1473,3 +1531,107 @@ osync_bool osync_member_delete_data(OSyncMember *member, OSyncChange *change)
 }
 
 /*@}*/
+
+
+/** Write the list of objtypes that had slow-sync set to a message */
+void osync_member_write_sink_info(OSyncMember *member, OSyncMessage *message)
+{
+	GList *obj = NULL;
+	for (osync_member_get_objtype_sinks(member, &obj, NULL); obj; obj = obj->next) {
+		OSyncObjTypeSink *sink = obj->data;
+		int slow;
+		slow = osync_member_get_slow_sync(member, sink->objtype->name);
+		osync_message_write_string(message, sink->objtype->name);
+		osync_message_write_int(message, sink->read);
+		osync_message_write_int(message, sink->write);
+		osync_message_write_int(message, sink->enabled);
+		osync_message_write_int(message, slow);
+	}
+	osync_message_write_string(message, NULL);
+}
+
+/** Read sink info for member
+ *
+ * @see osync_member_read_sink_info_full(), osync_member_read_sink_info()
+ */
+int __sync_member_read_sink_info(OSyncMember *member, OSyncMessage *message)
+{
+	char *objtypestr;
+	int r = 0;
+	for (;;) {
+		int read, write, enabled, slow;
+		osync_message_read_string(message, &objtypestr);
+		if (!objtypestr)
+			break;
+
+		osync_message_read_int(message, &read);
+		osync_message_read_int(message, &write);
+		osync_message_read_int(message, &enabled);
+		osync_message_read_int(message, &slow);
+
+		OSyncObjTypeSink *sink = osync_member_find_objtype_sink(member, objtypestr);
+		if (!sink)
+			continue;
+
+		sink->read = read;
+		sink->write = write;
+		sink->enabled = enabled;
+
+		if (slow) {
+			osync_member_set_slow_sync(member, objtypestr, TRUE);
+			r++;
+		}
+
+		free(objtypestr);
+	}
+	return r;
+}
+
+
+/** Read a list of objtypes that had slow-sync set, replacing old settings
+ *
+ * Please notice that this function will reset any old
+ * slow-sync setting that was set before. So, this should
+ * be used only for messages that is known to contain the
+ * complete slow-sync settings, not only for a member.
+ *
+ * i.e. this function may be used to read the slow-sync settings
+ * from the engine to osplugin, but not to read the slow-sync
+ * settings from osplugin to the engine, because osplugin doesn't
+ * know about the slow-sync settings of other members in the sync
+ * group.
+ */
+void osync_member_read_sink_info_full(OSyncMember *member, OSyncMessage *message)
+{
+	osync_group_reset_slow_sync(member->group, "data");
+	__sync_member_read_sink_info(member, message);
+}
+
+/** Read a list of objtypes that had slow-sync set, keeping old settings
+ *
+ * Please notice that this function won't reset
+ * the list of slow-sync settings, like
+ * osync_message_read_slow_sync_full_list(), but instead
+ * it will just set slow-sync for the objtypes received
+ * in the list.
+ *
+ * This function is supposed to be used when handling messages
+ * from osplugin to the engine, but NOT for messages from the engine
+ * to osplugin.
+ */
+void osync_member_read_sink_info(OSyncMember *member, OSyncMessage *message)
+{
+	int set_for_any;
+
+	set_for_any = __sync_member_read_sink_info(member, message);
+
+	if (set_for_any) {
+		/* FIXME: We must force slow-sync in "data" when some
+		 * objtype is marked to slow-sync
+		 *
+		 * (See ticket #1011)
+		 */
+		osync_member_set_slow_sync(member, "data", TRUE);
+	}
+}
+
