@@ -102,7 +102,7 @@ void _osync_format_set_commit(OSyncObjTypeTemplate *template, const char *format
 	OSyncObjFormatTemplate *format_template = NULL;
 	if (formatstr) {
 		OSyncObjFormatTemplate *format_template = osync_plugin_find_objformat_template(template, formatstr);
-		osync_assert(format_template, "Unable to set commit function. Did you forget to add the objformat?");
+		osync_assert_msg(format_template, "Unable to set commit function. Did you forget to add the objformat?");
 		format_template->commit_change = commit_change;
 	} else {
 		GList *f = NULL;
@@ -118,7 +118,7 @@ void _osync_format_set_access(OSyncObjTypeTemplate *template, const char *format
 	OSyncObjFormatTemplate *format_template = NULL;
 	if (formatstr) {
 		format_template = osync_plugin_find_objformat_template(template, formatstr);
-		osync_assert(format_template, "Unable to set commit function. Did you forget to add the objformat?");
+		osync_assert_msg(format_template, "Unable to set commit function. Did you forget to add the objformat?");
 		format_template->access = access;
 	} else {
 		GList *f = NULL;
@@ -134,7 +134,7 @@ void _osync_format_set_batch(OSyncObjTypeTemplate *template, const char *formats
 	OSyncObjFormatTemplate *format_template = NULL;
 	if (formatstr) {
 		format_template = osync_plugin_find_objformat_template(template, formatstr);
-		osync_assert(format_template, "Unable to set batch commit function. Did you forget to add the objformat?");
+		osync_assert_msg(format_template, "Unable to set batch commit function. Did you forget to add the objformat?");
 		format_template->batch_commit = batch;
 	} else {
 		GList *f = NULL;
@@ -291,9 +291,11 @@ osync_bool osync_module_load(OSyncEnv *env, const char *path, OSyncError **error
 	void (* fct_info)(OSyncEnv *env) = NULL;
 	void (** fct_infop)(OSyncEnv *env) = &fct_info;
 	if (!g_module_symbol(module, "get_info", (void **)fct_infop)) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get symbol from module %s: %s", path, g_module_error());
-		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-		return FALSE;
+		/* If there is no get_info symbol, the file must be a implementation library
+		 * for one of the other modules. So dont throw an error error since it has been
+		 * confusing users */
+		osync_trace(TRACE_EXIT, "%s: Not loading implementation library", __func__);
+		return TRUE;
 	}
 	env->modules = g_list_append(env->modules, module);
 	
@@ -445,6 +447,12 @@ void *osync_plugin_get_plugin_data(OSyncPlugin *plugin)
 	return plugin->info.plugin_data;
 }
 
+const char *osync_plugin_get_path(OSyncPlugin *plugin)
+{
+	g_assert(plugin);
+	return g_module_name(plugin->real_plugin);
+}
+
 /*! @brief Sets the commit function of a format
  * 
  * @param info Pointer to a plugin info struct to fill
@@ -459,7 +467,7 @@ void osync_plugin_set_commit_objformat(OSyncPluginInfo *info, const char *objtyp
 	
 	if (objtypestr) {
 		OSyncObjTypeTemplate *template = osync_plugin_find_objtype_template(info->plugin, objtypestr);
-		osync_assert(template, "Unable to accept objformat. Did you forget to add the objtype?");
+		osync_assert_msg(template, "Unable to accept objformat. Did you forget to add the objtype?");
 		_osync_format_set_commit(template, formatstr, commit_change);
 	} else {
 		GList *o = NULL;
@@ -483,9 +491,9 @@ void osync_plugin_set_access_objformat(OSyncPluginInfo *info, const char *objtyp
 	OSyncObjTypeTemplate *template = NULL;
 	
 	if (objtypestr) {
-		template = osync_plugin_find_objtype_template(info->plugin, objtypestr);
-		osync_assert(template, "Unable to accept objformat. Did you forget to add the objtype?");
-		_osync_format_set_access(template, formatstr, access);
+		//template = osync_plugin_find_objtype_template(info->plugin, objtypestr);
+		//osync_assert_msg(template, "Unable to accept objformat. Did you forget to add the objtype?");
+		//_osync_format_set_access(template, formatstr, access);
 	} else {
 		GList *o = NULL;
 		for (o = info->plugin->accepted_objtypes; o; o = o->next) {
@@ -506,9 +514,9 @@ void osync_plugin_set_access_objformat(OSyncPluginInfo *info, const char *objtyp
 void osync_plugin_set_read_objformat(OSyncPluginInfo *info, const char *objtypestr, const char *formatstr, OSyncFormatReadFn read)
 {
 	OSyncObjTypeTemplate *template = osync_plugin_find_objtype_template(info->plugin, objtypestr);
-	osync_assert(template, "Unable to accept objformat. Did you forget to add the objtype?");
+	osync_assert_msg(template, "Unable to accept objformat. Did you forget to add the objtype?");
 	OSyncObjFormatTemplate *format_template = osync_plugin_find_objformat_template(template, formatstr);
-	osync_assert(format_template, "Unable to set commit function. Did you forget to add the objformat?");
+	osync_assert_msg(format_template, "Unable to set commit function. Did you forget to add the objformat?");
 	format_template->read = read;
 }
 
@@ -526,7 +534,7 @@ void osync_plugin_set_batch_commit_objformat(OSyncPluginInfo *info, const char *
 	
 	if (objtypestr) {
 		template = osync_plugin_find_objtype_template(info->plugin, objtypestr);
-		osync_assert(template, "Unable to accept objformat. Did you forget to add the objtype?");
+		osync_assert_msg(template, "Unable to accept objformat. Did you forget to add the objtype?");
 		_osync_format_set_batch(template, formatstr, batch);
 	} else {
 		GList *o = NULL;
@@ -548,9 +556,9 @@ void osync_plugin_set_batch_commit_objformat(OSyncPluginInfo *info, const char *
 void osync_plugin_set_committed_all_objformat(OSyncPluginInfo *info, const char *objtypestr, const char *formatstr, OSyncFormatCommittedAllFn committed_all)
 {
 	OSyncObjTypeTemplate *template = osync_plugin_find_objtype_template(info->plugin, objtypestr);
-	osync_assert(template, "Unable to accept objformat. Did you forget to add the objtype?");
+	osync_assert_msg(template, "Unable to accept objformat. Did you forget to add the objtype?");
 	OSyncObjFormatTemplate *format_template = osync_plugin_find_objformat_template(template, formatstr);
-	osync_assert(format_template, "Unable to set committed_all function. Did you forget to add the objformat?");
+	osync_assert_msg(format_template, "Unable to set committed_all function. Did you forget to add the objformat?");
 	format_template->committed_all = committed_all;
 }
 
@@ -584,7 +592,7 @@ void osync_plugin_accept_objtype(OSyncPluginInfo *info, const char *objtypestr)
 void osync_plugin_accept_objformat(OSyncPluginInfo *info, const char *objtypestr, const char *formatstr, const char *extension)
 {
 	OSyncObjTypeTemplate *template = osync_plugin_find_objtype_template(info->plugin, objtypestr);
-	osync_assert(template, "Unable to accept objformat. Did you forget to add the objtype?");
+	osync_assert_msg(template, "Unable to accept objformat. Did you forget to add the objtype?");
 	OSyncObjFormatTemplate *format_template = g_malloc0(sizeof(OSyncObjFormatTemplate));
 	format_template->name = g_strdup(formatstr);
 	if (extension)
