@@ -72,7 +72,7 @@ static OSyncMappingEntry *_osync_change_clone(OSyncEngine *engine, OSyncMapping
 	return newentry;
 }
 
-static osync_bool _osync_change_elevate(OSyncEngine *engine, OSyncChange *change, int level)
+osync_bool osync_change_elevate(OSyncEngine *engine, OSyncChange *change, int level)
 {
 	osync_debug("MAP", 3, "elevating change %s (%p) to level %i", osync_change_get_uid(change), change, level);
 	int i = 0;
@@ -85,7 +85,7 @@ static osync_bool _osync_change_elevate(OSyncEngine *engine, OSyncChange *change
 	return TRUE;
 }
 
-static osync_bool _osync_change_check_level(OSyncEngine *engine, OSyncMappingEntry *entry)
+osync_bool osync_change_check_level(OSyncEngine *engine, OSyncMappingEntry *entry)
 {
 	GList *c;
 	osync_debug("MAP", 3, "checking level for change %s (%p)", osync_change_get_uid(entry->change), entry);
@@ -358,10 +358,10 @@ void osengine_mapping_duplicate(OSyncEngine *engine, OSyncMapping *dupe_mapping)
 		 * or a (2) to the change uid. We then check if there is already
 		 * another change on this level and if there is, we elevate again */
 		do {
-			if (!_osync_change_elevate(engine, newentry->change, 1))
+			if (!osync_change_elevate(engine, newentry->change, 1))
 				break;
 			elevation += 1;
-		} while (!_osync_change_check_level(engine, newentry));
+		} while (!osync_change_check_level(engine, newentry));
 		OSyncError *error = NULL;
 		osync_change_save(newentry->change, TRUE, &error);
 		
@@ -370,7 +370,7 @@ void osengine_mapping_duplicate(OSyncEngine *engine, OSyncMapping *dupe_mapping)
 		 * to be different from the master of the mapping to duplicate */
 		while ((next_entry = _osync_find_next_same(dupe_mapping, first_diff_entry))) {
 			newentry = _osync_change_clone(engine, new_mapping, first_diff_entry);
-			_osync_change_elevate(engine, newentry->change, elevation);
+			osync_change_elevate(engine, newentry->change, elevation);
 			osengine_mappingentry_update(orig_entry, next_entry->change);
 			osync_change_save(next_entry->change, TRUE, NULL);
 		}
@@ -417,9 +417,15 @@ void osengine_mapping_solve(OSyncEngine *engine, OSyncMapping *mapping, OSyncCha
  * @param mapping The mapping to ignore
  * 
  */
-void osengine_mapping_ignore_conflict(OSyncEngine *engine, OSyncMapping *mapping)
+osync_bool osengine_mapping_ignore_conflict(OSyncEngine *engine, OSyncMapping *mapping, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, engine, mapping);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, error);
+	
+	if (!osengine_mapping_ignore_supported(engine, mapping)) {
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Ignore is not supported for this mapping");
+		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+		return FALSE;
+	}
 	
 	GList *e = NULL;
 	for (e = mapping->entries; e; e = e->next) {
@@ -436,6 +442,46 @@ void osengine_mapping_ignore_conflict(OSyncEngine *engine, OSyncMapping *mapping
 	osync_flag_set(mapping->cmb_synced);
 	osync_flag_set(mapping->cmb_has_info);
 	osync_trace(TRACE_EXIT, "%s", __func__);
+	return TRUE;
+}
+
+/** @brief Checks if a conflict can be ignore
+ * 
+ * To be able to ignore a conflict, you opensync must be able to read
+ * the changes of the conflict again during the next synchronization. This must be done
+ * even if they are not reported by the plugin. Therefore, all plugins should provide
+ * a "read" method. If there is a member in the engine's group that does not have this
+ * method (either since it is not possible to implement one or since it has not been done
+ * yet), this function will return FALSE.
+ *
+ * @param engine The engine
+ * @param mapping The mapping to check
+ * @returns TRUE if conflicts can be ignored, FALSE otherwise
+ */
+osync_bool osengine_mapping_ignore_supported(OSyncEngine *engine, OSyncMapping *mapping)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, engine, mapping);
+	
+	int i, count = 0;
+	OSyncChange *change = NULL;
+	OSyncMember *member = NULL;
+	OSyncObjType *objtype = NULL;
+
+	count = osengine_mapping_num_changes(mapping);
+	for (i = 0; i < count; ++i) {
+		change = osengine_mapping_nth_change(mapping, i);
+		objtype = osync_change_get_objtype(change);
+
+		member = osync_change_get_member(change);
+
+		if (!osync_member_has_read_function(member, objtype)) {
+			osync_trace(TRACE_EXIT, "%s: Ignore NOT supported", __func__);
+			return FALSE;
+		}
+	}
+	
+	osync_trace(TRACE_EXIT, "%s: Ignore supported", __func__);
+	return TRUE;
 }
 
 /** @brief Solves a mapping by choosing the entry that was last modified
